# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
from __future__ import absolute_import

import pprint
import signal
import time
import traceback
import subprocess
from threading import Event

import mozcrash
import psutil
from mozlog import get_proxy_logger
from mozprocess import ProcessHandler
from talos.utils import TalosError

LOG = get_proxy_logger()


class ProcessContext(object):
    """
    Store useful results of the browser execution.
    """
    def __init__(self):
        self.output = None
        self.process = None

    @property
    def pid(self):
        return self.process and self.process.pid

    def kill_process(self):
        """
        Kill the process, returning the exit code or None if the process
        is already finished.
        """
        if self.process and self.process.is_running():
            LOG.debug("Terminating %s" % self.process)
            try:
                self.process.terminate()
            except psutil.NoSuchProcess:
                procs = self.process.children()
                for p in procs:
                    c = ProcessContext()
                    c.process = p
                    c.kill_process()
                return self.process.returncode
            try:
                return self.process.wait(3)
            except psutil.TimeoutExpired:
                self.process.kill()
                # will raise TimeoutExpired if unable to kill
                return self.process.wait(3)


class Reader(object):
    def __init__(self, event):
        self.output = []
        self.got_end_timestamp = False
        self.got_timeout = False
        self.timeout_message = ''
        self.got_error = False
        self.event = event
        self.proc = None

    def __call__(self, line):
        if line.find('__endTimestamp') != -1:
            self.got_end_timestamp = True
            self.event.set()
        elif line == 'TART: TIMEOUT':
            self.got_timeout = True
            self.timeout_message = 'TART'
            self.event.set()
        elif line.startswith('TEST-UNEXPECTED-FAIL | '):
            self.got_error = True
            self.event.set()

        if not (line.startswith('JavaScript error:') or
                line.startswith('JavaScript warning:')):
            LOG.process_output(self.proc.pid, line)
            self.output.append(line)


def run_browser(command, minidump_dir, timeout=None, on_started=None,
                debug=None, debugger=None, debugger_args=None, **kwargs):
    """
    Run the browser using the given `command`.

    After the browser prints __endTimestamp, we give it 5
    seconds to quit and kill it if it's still alive at that point.

    Note that this method ensure that the process is killed at
    the end. If this is not possible, an exception will be raised.

    :param command: the commad (as a string list) to run the browser
    :param minidump_dir: a path where to extract minidumps in case the
                         browser hang. This have to be the same value
                         used in `mozcrash.check_for_crashes`.
    :param timeout: if specified, timeout to wait for the browser before
                    we raise a :class:`TalosError`
    :param on_started: a callback that can be used to do things just after
                       the browser has been started. The callback must takes
                       an argument, which is the psutil.Process instance
    :param kwargs: additional keyword arguments for the :class:`ProcessHandler`
                   instance

    Returns a ProcessContext instance, with available output and pid used.
    """

    debugger_info = find_debugger_info(debug, debugger, debugger_args)
    if debugger_info is not None:
        return run_in_debug_mode(command, debugger_info,
                                 on_started=on_started, env=kwargs.get('env'))

    context = ProcessContext()
    first_time = int(time.time()) * 1000
    wait_for_quit_timeout = 5
    event = Event()
    reader = Reader(event)

    LOG.info("Using env: %s" % pprint.pformat(kwargs['env']))

    kwargs['storeOutput'] = False
    kwargs['processOutputLine'] = reader
    kwargs['onFinish'] = event.set
    proc = ProcessHandler(command, **kwargs)
    reader.proc = proc
    proc.run()

    LOG.process_start(proc.pid, ' '.join(command))
    try:
        context.process = psutil.Process(proc.pid)
        if on_started:
            on_started(context.process)
        # wait until we saw __endTimestamp in the proc output,
        # or the browser just terminated - or we have a timeout
        if not event.wait(timeout):
            LOG.info("Timeout waiting for test completion; killing browser...")
            # try to extract the minidump stack if the browser hangs
            mozcrash.kill_and_get_minidump(proc.pid, minidump_dir)
            raise TalosError("timeout")
        if reader.got_end_timestamp:
            for i in range(1, wait_for_quit_timeout):
                if proc.wait(1) is not None:
                    break
            if proc.poll() is None:
                LOG.info(
                    "Browser shutdown timed out after {0} seconds, terminating"
                    " process.".format(wait_for_quit_timeout)
                )
        elif reader.got_timeout:
            raise TalosError('TIMEOUT: %s' % reader.timeout_message)
        elif reader.got_error:
            raise TalosError("unexpected error")
    finally:
        # this also handle KeyboardInterrupt
        # ensure early the process is really terminated
        return_code = None
        try:
            return_code = context.kill_process()
            if return_code is None:
                return_code = proc.wait(1)
        except Exception:
            # Maybe killed by kill_and_get_minidump(), maybe ended?
            LOG.info("Unable to kill process")
            LOG.info(traceback.format_exc())

    reader.output.append(
        "__startBeforeLaunchTimestamp%d__endBeforeLaunchTimestamp"
        % first_time)
    reader.output.append(
        "__startAfterTerminationTimestamp%d__endAfterTerminationTimestamp"
        % (int(time.time()) * 1000))

    if return_code is not None:
        LOG.process_exit(proc.pid, return_code)
    else:
        LOG.debug("Unable to detect exit code of the process %s." % proc.pid)
    context.output = reader.output
    return context


def find_debugger_info(debug, debugger, debugger_args):
    debuggerInfo = None
    if debug or debugger or debugger_args:
        import mozdebug

        if not debugger:
            # No debugger name was provided. Look for the default ones on
            # current OS.
            debugger = mozdebug.get_default_debugger_name(mozdebug.DebuggerSearch.KeepLooking)

        debuggerInfo = None
        if debugger:
            debuggerInfo = mozdebug.get_debugger_info(debugger, debugger_args)

        if debuggerInfo is None:
            raise TalosError('Could not find a suitable debugger in your PATH.')

    return debuggerInfo


def run_in_debug_mode(command, debugger_info, on_started=None, env=None):
    signal.signal(signal.SIGINT, lambda sigid, frame: None)
    context = ProcessContext()
    command_under_dbg = [debugger_info.path] + debugger_info.args + command

    ttest_process = subprocess.Popen(command_under_dbg, env=env)

    context.process = psutil.Process(ttest_process.pid)
    if on_started:
        on_started(context.process)

    return_code = ttest_process.wait()

    if return_code is not None:
        LOG.process_exit(ttest_process.pid, return_code)
    else:
        LOG.debug("Unable to detect exit code of the process %s." % ttest_process.pid)

    return context



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
from __future__ import absolute_import, print_function

import argparse
import os

from mozlog.commandline import add_logging_group


class _StopAction(argparse.Action):
    def __init__(self, option_strings, dest=argparse.SUPPRESS,
                 default=argparse.SUPPRESS, help=None):
        super(_StopAction, self).__init__(
            option_strings=option_strings,
            dest=dest,
            default=default,
            nargs=0,
            help=help)


class _ListTests(_StopAction):
    def __call__(self, parser, namespace, values, option_string=None):
        from talos import test
        print('Available tests:')
        print('================\n')
        test_class_names = [
            (test_class.name(), test_class.description())
            for test_class in test.test_dict().itervalues()
        ]
        test_class_names.sort()
        for name, description in test_class_names:
            print(name)
            print('-'*len(name))
            print(description)
            print  # Appends a single blank line to the end
        parser.exit()


class _ListSuite(_StopAction):
    def __call__(self, parser, namespace, values, option_string=None):
        from talos.config import suites_conf
        print('Available suites:')
        conf = suites_conf()
        max_suite_name = max([len(s) for s in conf])
        pattern = " %%-%ds (%%s)" % max_suite_name
        for name in conf:
            print(pattern % (name, ':'.join(conf[name]['tests'])))
        print
        parser.exit()


def create_parser(mach_interface=False):
    parser = argparse.ArgumentParser()
    add_arg = parser.add_argument

    if not mach_interface:
        add_arg('-e', '--executablePath', required=True, dest="browser_path",
                help="path to executable we are testing")
    add_arg('-t', '--title', default='qm-pxp01',
            help="Title of the test run")
    add_arg('--browserWait', dest='browser_wait', default=5, type=int,
            help="Amount of time allowed for the browser to cleanly close")
    add_arg('-a', '--activeTests',
            help="List of tests to run, separated by ':' (ex. damp:tart)")
    add_arg('--suite',
            help="Suite to use (instead of --activeTests)")
    add_arg('--subtests',
            help="Name of the subtest(s) to run (works only on DAMP)")
    add_arg('--mainthread', action='store_true',
            help="Collect mainthread IO data from the browser by setting"
                 " an environment variable")
    add_arg("--mozAfterPaint", action='store_true', dest="tpmozafterpaint",
            help="wait for MozAfterPaint event before recording the time")
    add_arg("--firstPaint", action='store_true', dest="firstpaint",
            help="Also report the first paint value in supported tests")
    add_arg("--useHero", action='store_true', dest="tphero",
            help="use Hero elementtiming attribute to record the time")
    add_arg("--userReady", action='store_true', dest="userready",
            help="Also report the user ready value in supported tests")
    add_arg('--spsProfile', action="store_true", dest="gecko_profile",
            help="(Deprecated - Use --gecko-profile instead.) Profile the "
                 "run and output the results in $MOZ_UPLOAD_DIR.")
    add_arg('--spsProfileInterval', dest='gecko_profile_interval', type=float,
            help="(Deprecated - Use --gecko-profile-interval instead.) How "
                 "frequently to take samples (ms)")
    add_arg('--spsProfileEntries', dest="gecko_profile_entries", type=int,
            help="(Deprecated - Use --gecko-profile-entries instead.) How "
                 "many samples to take with the profiler")
    add_arg('--geckoProfile', action="store_true", dest="gecko_profile",
            help=argparse.SUPPRESS)
    add_arg('--geckoProfileInterval', dest='gecko_profile_interval', type=float,
            help=argparse.SUPPRESS)
    add_arg('--geckoProfileEntries', dest="gecko_profile_entries", type=int,
            help=argparse.SUPPRESS)
    add_arg('--gecko-profile', action="store_true", dest="gecko_profile",
            help="Profile the run and output the results in $MOZ_UPLOAD_DIR. "
                 "After talos is finished, profiler.firefox.com will be launched in "
                 "Firefox so you can analyze the local profiles. To disable "
                 "auto-launching of profiler.firefox.com set the "
                 "DISABLE_PROFILE_LAUNCH=1 env var.")
    add_arg('--gecko-profile-interval', dest='gecko_profile_interval', type=float,
            help="How frequently to take samples (ms)")
    add_arg('--gecko-profile-entries', dest="gecko_profile_entries", type=int,
            help="How many samples to take with the profiler")
    add_arg('--extension', dest='extensions', action='append',
            default=['${talos}/talos-powers'],
            help="Extension to install while running")
    add_arg('--fast', action='store_true',
            help="Run tp tests as tp_fast")
    add_arg('--symbolsPath', dest='symbols_path',
            help="Path to the symbols for the build we are testing")
    add_arg('--xperf_path',
            help="Path to windows performance tool xperf.exe")
    add_arg('--test_timeout', type=int, default=1200,
            help="Time to wait for the browser to output to the log file")
    add_arg('--errorFile', dest='error_filename',
            default=os.path.abspath('browser_failures.txt'),
            help="Filename to store the errors found during the test."
                 " Currently used for xperf only.")
    add_arg('--setpref', action='append', default=[], dest="extraPrefs",
            metavar="PREF=VALUE",
            help="defines an extra user preference")
    add_arg("--firstNonBlankPaint", action='store_true', dest="fnbpaint",
            help="Wait for firstNonBlankPaint event before recording the time")
    add_arg("--pdfPaint", action='store_true', dest="pdfpaint",
            help="Wait for the first page of a PDF to be rendered")
    add_arg('--webServer', dest='webserver',
            help="DEPRECATED")
    if not mach_interface:
        add_arg('--develop', action='store_true', default=False,
                help="useful for running tests on a developer machine."
                     " Doesn't upload to the graph servers.")
    add_arg("--cycles", type=int,
            help="number of browser cycles to run")
    add_arg("--tpmanifest",
            help="manifest file to test")
    add_arg('--tpcycles', type=int,
            help="number of pageloader cycles to run")
    add_arg('--tptimeout', type=int,
            help='number of milliseconds to wait for a load event after'
                 ' calling loadURI before timing out')
    add_arg('--tppagecycles', type=int,
            help='number of pageloader cycles to run for each page in'
                 ' the manifest')
    add_arg('--no-download', action="store_true", dest="no_download",
            help="Do not download the talos test pagesets")
    add_arg('--sourcestamp',
            help='Specify the hg revision or sourcestamp for the changeset'
                 ' we are testing.  This will use the value found in'
                 ' application.ini if it is not specified.')
    add_arg('--repository',
            help='Specify the url for the repository we are testing. '
                 'This will use the value found in application.ini if'
                 ' it is not specified.')
    add_arg('--framework',
            help='Will post to the specified framework for Perfherder. '
                 'Default "talos".  Used primarily for experiments on '
                 'new platforms')
    add_arg('--print-tests', action=_ListTests,
            help="print available tests")
    add_arg('--print-suites', action=_ListSuite,
            help="list available suites")
    add_arg('--no-upload-results', action="store_true",
            dest='no_upload_results',
            help="If given, it disables uploading of talos results.")
    add_arg('--stylo-threads', type=int,
            dest='stylothreads',
            help='If given, run Stylo with a certain number of threads')
    add_arg('--profile', type=str, default=None,
            help="Downloads a profile from TaskCluster and uses it")
    debug_options = parser.add_argument_group('Command Arguments for debugging')
    debug_options.add_argument('--debug', action='store_true',
                               help='Enable the debugger. Not specifying a --debugger option will'
                                    'result in the default debugger being used.')
    debug_options.add_argument('--debugger', default=None,
                               help='Name of debugger to use.')
    debug_options.add_argument('--debugger-args', default=None, metavar='params',
                               help='Command-line arguments to pass to the debugger itself; split'
                                    'as the Bourne shell would.')
    add_arg('--code-coverage', action="store_true",
            dest='code_coverage',
            help='Remove any existing ccov gcda output files after browser'
                 ' initialization but before starting the tests. NOTE:'
                 ' Currently only supported in production.')
    add_arg('--enable-webrender', action="store_true", default=False,
            help="Enable the WebRender compositor in Gecko.")
    add_arg('--enable-fission', action="store_true", default=False,
            help="Enable Fission (site isolation) in Gecko.")

    add_logging_group(parser)
    return parser


def parse_args(argv=None):
    parser = create_parser()
    return parser.parse_args(argv)



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this file,
# You can obtain one at http://mozilla.org/MPL/2.0/.
from __future__ import absolute_import, print_function

import copy
import os
import sys

from mozlog.commandline import setup_logging
from talos import utils, test
from talos.cmdline import parse_args


class ConfigurationError(Exception):
    pass


DEFAULTS = dict(
    # args to pass to browser
    extra_args=[],
    buildid='testbuildid',
    init_url='getInfo.html',
    env={'NO_EM_RESTART': '1'},
    # base data for all tests
    basetest=dict(
        cycles=1,
        profile_path='${talos}/base_profile',
        responsiveness=False,
        gecko_profile=False,
        gecko_profile_interval=1,
        gecko_profile_entries=100000,
        resolution=1,
        mainthread=False,
        shutdown=False,
        timeout=3600,
        tpchrome=True,
        tpcycles=10,
        tpmozafterpaint=False,
        tphero=False,
        pdfpaint=True,
        fnbpaint=False,
        firstpaint=False,
        format_pagename=True,
        userready=False,
        testeventmap=[],
        base_vs_ref=False,
        tppagecycles=1,
        tploadnocache=False,
        tpscrolltest=False,
        win_counters=[],
        w7_counters=[],
        linux_counters=[],
        mac_counters=[],
        xperf_counters=[],
        setup=None,
        cleanup=None,
        preferences={},
    ),
)


# keys to generated self.config that are global overrides to tests
GLOBAL_OVERRIDES = (
    'cycles',
    'gecko_profile',
    'gecko_profile_interval',
    'gecko_profile_entries',
    'tpcycles',
    'tppagecycles',
    'tpmanifest',
    'tptimeout',
    'tpmozafterpaint',
    'tphero',
    'fnbpaint',
    'pdfpaint',
    'firstpaint',
    'userready',
)


CONF_VALIDATORS = []


def validator(func):
    """
    A decorator that register configuration validators to execute against the
    configuration.

    They will be executed in the order of declaration.
    """
    CONF_VALIDATORS.append(func)
    return func


def convert_url(config, url):
    webserver = config['webserver']
    if not webserver:
        return url

    if '://' in url:
        # assume a fully qualified url
        return url

    if '.html' in url:
        url = 'http://%s/%s' % (webserver, url)

    return url


@validator
def fix_xperf(config):
    # BBB: remove doubly-quoted xperf values from command line
    # (needed for buildbot)
    # https://bugzilla.mozilla.org/show_bug.cgi?id=704654#c43
    win7_path = 'c:/Program Files/Microsoft Windows Performance Toolkit/xperf.exe'
    if config['xperf_path']:
        xperf_path = config['xperf_path']
        quotes = ('"', "'")
        for quote in quotes:
            if xperf_path.startswith(quote) and xperf_path.endswith(quote):
                config['xperf_path'] = xperf_path[1:-1]
                break
        if not os.path.exists(config['xperf_path']):
            # look for old win7 path
            if not os.path.exists(win7_path):
                raise ConfigurationError(
                    "xperf.exe cannot be found at the path specified")
            config['xperf_path'] = win7_path


@validator
def set_webserver(config):
    # pick a free port
    import socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.bind(('', 0))
    port = sock.getsockname()[1]
    sock.close()

    config['webserver'] = '127.0.0.1:%d' % port


@validator
def update_prefs(config):
    config.setdefault('preferences', {})

    # update prefs from command line
    prefs = config.pop('extraPrefs')
    if prefs:
        for arg in prefs:
            k, v = arg.split('=', 1)
            config['preferences'][k] = utils.parse_pref(v)


@validator
def fix_init_url(config):
    if 'init_url' in config:
        config['init_url'] = convert_url(config, config['init_url'])


@validator
def determine_local_symbols_path(config):
    if 'symbols_path' not in config:
        return

    # use objdir/dist/crashreporter-symbols for symbolsPath if none provided
    if not config['symbols_path'] and \
       config['develop'] and \
       'MOZ_DEVELOPER_OBJ_DIR' in os.environ:
        config['symbols_path'] = os.path.join(os.environ['MOZ_DEVELOPER_OBJ_DIR'],
                                              'dist',
                                              'crashreporter-symbols')


def get_counters(config):
    counters = set()
    return counters


def get_active_tests(config):
    activeTests = config.pop('activeTests').strip().split(':')

    # ensure tests are available
    availableTests = test.test_dict()
    if not set(activeTests).issubset(availableTests):
        missing = [i for i in activeTests
                   if i not in availableTests]
        raise ConfigurationError("No definition found for test(s): %s"
                                 % missing)

    return activeTests


def get_global_overrides(config):
    global_overrides = {}
    for key in GLOBAL_OVERRIDES:
        # get global overrides for all tests
        value = config[key]
        if value is not None:
            global_overrides[key] = value
        if key != 'gecko_profile':
            config.pop(key)

    return global_overrides


def build_manifest(config, manifestName):
    # read manifest lines
    with open(manifestName, 'r') as fHandle:
        manifestLines = fHandle.readlines()

    # write modified manifest lines
    with open(manifestName + '.develop', 'w') as newHandle:
        for line in manifestLines:
            newline = line.replace('localhost', config['webserver'])
            newline = newline.replace('page_load_test', 'tests')
            newHandle.write(newline)

    newManifestName = manifestName + '.develop'

    # return new manifest
    return newManifestName


def get_test(config, global_overrides, counters, test_instance):
    mozAfterPaint = getattr(test_instance, 'tpmozafterpaint', None)
    hero = getattr(test_instance, 'tphero', None)
    firstPaint = getattr(test_instance, 'firstpaint', None)
    userReady = getattr(test_instance, 'userready', None)
    firstNonBlankPaint = getattr(test_instance, 'fnbpaint', None)
    pdfPaint = getattr(test_instance, 'pdfpaint', None)

    test_instance.update(**global_overrides)

    # update original value of mozAfterPaint, this could be 'false',
    # so check for None
    if mozAfterPaint is not None:
        test_instance.tpmozafterpaint = mozAfterPaint
    if firstNonBlankPaint is not None:
        test_instance.fnbpaint = firstNonBlankPaint
    if firstPaint is not None:
        test_instance.firstpaint = firstPaint
    if userReady is not None:
        test_instance.userready = userReady
    if hero is not None:
        test_instance.tphero = hero
    if pdfPaint is not None:
        test_instance.pdfpaint = pdfPaint

    # fix up url
    url = getattr(test_instance, 'url', None)
    if url:
        test_instance.url = utils.interpolate(convert_url(config, url))

    # fix up tpmanifest
    tpmanifest = getattr(test_instance, 'tpmanifest', None)
    if tpmanifest:
        test_instance.tpmanifest = \
            build_manifest(config, utils.interpolate(tpmanifest))

    # add any counters
    if counters:
        keys = ('linux_counters', 'mac_counters',
                'win_counters', 'w7_counters', 'xperf_counters')
        for key in keys:
            if key not in test_instance.keys:
                # only populate attributes that will be output
                continue
            if not isinstance(getattr(test_instance, key, None), list):
                setattr(test_instance, key, [])
            _counters = getattr(test_instance, key)
            _counters.extend([counter for counter in counters
                              if counter not in _counters])
    return dict(test_instance.items())


@validator
def tests(config):
    counters = get_counters(config)
    global_overrides = get_global_overrides(config)
    activeTests = get_active_tests(config)
    test_dict = test.test_dict()

    tests = []
    for test_name in activeTests:
        test_class = test_dict[test_name]
        tests.append(get_test(config, global_overrides, counters,
                              test_class()))
    config['tests'] = tests


def get_browser_config(config):
    required = ('extensions', 'browser_path', 'browser_wait',
                'extra_args', 'buildid', 'env', 'init_url', 'webserver')
    optional = {'bcontroller_config': '${talos}/bcontroller.json',
                'child_process': 'plugin-container',
                'debug': False,
                'debugger': None,
                'debugger_args': None,
                'develop': False,
                'enable_webrender': False,
                'enable_fission': False,
                'process': '',
                'framework': 'talos',
                'repository': None,
                'sourcestamp': None,
                'symbols_path': None,
                'test_timeout': 1200,
                'xperf_path': None,
                'error_filename': None,
                'no_upload_results': False,
                'stylothreads': 0,
                'subtests': None,
                'preferences': {},
                }
    browser_config = dict(title=config['title'])
    browser_config.update(dict([(i, config[i]) for i in required]))
    browser_config.update(dict([(i, config.get(i, j))
                          for i, j in optional.items()]))
    return browser_config


def suites_conf():
    import json
    with open(os.path.join(os.path.dirname(utils.here),
                           'talos.json')) as f:
        return json.load(f)['suites']


def get_config(argv=None):
    argv = argv or sys.argv[1:]
    cli_opts = parse_args(argv=argv)
    if cli_opts.suite:
        # read the suite config, update the args
        try:
            suite_conf = suites_conf()[cli_opts.suite]
        except KeyError:
            raise ConfigurationError('No such suite: %r' % cli_opts.suite)
        argv += ['-a', ':'.join(suite_conf['tests'])]
        # talos_options in the suite config should not override command line
        # options, so we prepend argv with talos_options so that, when parsed,
        # the command line options will clobber the suite config options.
        argv = suite_conf.get('talos_options', []) + argv
        # args needs to be reparsed now
    elif not cli_opts.activeTests:
        raise ConfigurationError('--activeTests or --suite required!')

    cli_opts = parse_args(argv=argv)
    setup_logging("talos", cli_opts, {"tbpl": sys.stdout})
    config = copy.deepcopy(DEFAULTS)
    config.update(cli_opts.__dict__)
    for validate in CONF_VALIDATORS:
        validate(config)
    # remove None Values
    for k, v in config.items():
        if v is None:
            del config[k]
    return config


def get_configs(argv=None):
    config = get_config(argv=argv)
    browser_config = get_browser_config(config)
    return config, browser_config


if __name__ == '__main__':
    cfgs = get_configs()
    print(cfgs[0])
    print
    print(cfgs[1])



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""A generic means of running an URL based browser test
   follows the following steps
     - creates a profile
     - tests the profile
     - gets metrics for the current test environment
     - loads the url
     - collects info on any counters while test runs
     - waits for a 'dump' from the browser
"""
from __future__ import absolute_import, print_function

import json
import os
import platform
import shutil
import subprocess
import sys
import time

import mozcrash
import mozfile

from mozlog import get_proxy_logger
from talos.cmanager import CounterManagement
from talos.ffsetup import FFSetup
from talos.talos_process import run_browser
from talos import utils
from talos import results
from talos import talosconfig
from talos.utils import TalosCrash, TalosError, TalosRegression, run_in_debug_mode

LOG = get_proxy_logger()


class TTest(object):
    def check_for_crashes(self, browser_config, minidump_dir, test_name):
        # check for minidumps
        found = mozcrash.check_for_crashes(minidump_dir,
                                           browser_config['symbols_path'],
                                           test_name=test_name)
        mozfile.remove(minidump_dir)

        if found:
            raise TalosCrash('Found crashes after test run, terminating test')

    def runTest(self, browser_config, test_config):
        """
            Runs an url based test on the browser as specified in the
            browser_config dictionary

        Args:
            browser_config:  Dictionary of configuration options for the
                             browser (paths, prefs, etc)
            test_config   :  Dictionary of configuration for the given
                             test (url, cycles, counters, etc)

        """

        with FFSetup(browser_config, test_config) as setup:
            return self._runTest(browser_config, test_config, setup)

    @staticmethod
    def _get_counter_prefix():
        if platform.system() == 'Linux':
            return 'linux'
        elif platform.system() in ('Windows', 'Microsoft'):
            if '6.1' in platform.version():  # w7
                return 'w7'
            elif '6.2' in platform.version():  # w8
                return 'w8'
            # Bug 1264325 - FIXME: with python 2.7.11: reports win8 instead of 8.1
            elif '6.3' in platform.version():
                return 'w8'
            # Bug 1264325 - FIXME: with python 2.7.11: reports win8 instead of 10
            elif '10.0' in platform.version():
                return 'w8'
            else:
                raise TalosError('unsupported windows version')
        elif platform.system() == 'Darwin':
            return 'mac'

    def _runTest(self, browser_config, test_config, setup):
        minidump_dir = os.path.join(setup.profile_dir, 'minidumps')
        counters = test_config.get('%s_counters' % self._get_counter_prefix(), [])
        resolution = test_config['resolution']

        # add the mainthread_io to the environment variable, as defined
        # in test.py configs
        here = os.path.dirname(os.path.realpath(__file__))
        if test_config['mainthread']:
            mainthread_io = os.path.join(here, 'mainthread_io.log')
            setup.env['MOZ_MAIN_THREAD_IO_LOG'] = mainthread_io

        # Stylo is on by default
        setup.env['STYLO_FORCE_ENABLED'] = '1'

        # During the Stylo transition, measure different number of threads
        if browser_config.get('stylothreads', 0) > 0:
            setup.env['STYLO_THREADS'] = str(browser_config['stylothreads'])

        if browser_config['enable_webrender']:
            setup.env['MOZ_WEBRENDER'] = '1'
            setup.env['MOZ_ACCELERATED'] = '1'
        else:
            setup.env['MOZ_WEBRENDER'] = '0'

        # set url if there is one (i.e. receiving a test page, not a manifest/pageloader test)
        if test_config.get('url', None) is not None:
            test_config['url'] = utils.interpolate(
                test_config['url'],
                profile=setup.profile_dir,
                firefox=browser_config['browser_path']
            )
        else:
            setup.env['MOZ_USE_PAGELOADER'] = '1'

        # setup global (cross-cycle) responsiveness counters
        global_counters = {}
        if browser_config.get('xperf_path'):
            for c in test_config.get('xperf_counters', []):
                global_counters[c] = []

        if test_config.get('responsiveness') and \
           platform.system() != 'Darwin':
            # ignore osx for now as per bug 1245793
            setup.env['MOZ_INSTRUMENT_EVENT_LOOP'] = '1'
            setup.env['MOZ_INSTRUMENT_EVENT_LOOP_THRESHOLD'] = '20'
            setup.env['MOZ_INSTRUMENT_EVENT_LOOP_INTERVAL'] = '10'
            global_counters['responsiveness'] = []

        setup.env['JSGC_DISABLE_POISONING'] = '1'
        setup.env['MOZ_DISABLE_NONLOCAL_CONNECTIONS'] = '1'

        # instantiate an object to hold test results
        test_results = results.TestResults(
            test_config,
            global_counters,
            browser_config.get('framework')
        )

        for i in range(test_config['cycles']):
            time.sleep(0.25)
            LOG.info('Running cycle %d/%d for %s test...'
                     % (i+1, test_config['cycles'], test_config['name']))

            # remove the browser  error file
            mozfile.remove(browser_config['error_filename'])

            # individual tests can have different frameworks
            # TODO: ensure that we don't run >1 test with custom frameworks
            if test_config.get('perfherder_framework', None) is not None:
                test_results.framework = test_config['perfherder_framework']

            # reinstall any file whose stability we need to ensure across
            # the cycles
            if test_config.get('reinstall', ''):
                for keep in test_config['reinstall']:
                    origin = os.path.join(test_config['profile_path'],
                                          keep)
                    dest = os.path.join(setup.profile_dir, keep)
                    LOG.debug('Reinstalling %s on top of %s'
                              % (origin, dest))
                    shutil.copy(origin, dest)

            # Run the test
            timeout = test_config.get('timeout', 7200)  # 2 hours default
            if setup.gecko_profile:
                # When profiling, give the browser some extra time
                # to dump the profile.
                timeout += 5 * 60
                # store profiling info for pageloader; too late to add it as browser pref
                setup.env["TPPROFILINGINFO"] = json.dumps(setup.gecko_profile.profiling_info)

            command_args = utils.GenerateBrowserCommandLine(
                browser_config['browser_path'],
                browser_config['extra_args'],
                setup.profile_dir,
                test_config['url'],
                profiling_info=(setup.gecko_profile.profiling_info
                                if setup.gecko_profile else None)
            )

            mainthread_error_count = 0
            if test_config['setup']:
                # Generate bcontroller.json for xperf
                talosconfig.generateTalosConfig(command_args,
                                                browser_config,
                                                test_config)
                subprocess.call(
                    ['python'] + test_config['setup'].split(),
                )

            counter_management = None
            if counters:
                counter_management = CounterManagement(
                    browser_config['process'],
                    counters,
                    resolution
                )

            try:
                pcontext = run_browser(
                    command_args,
                    minidump_dir,
                    timeout=timeout,
                    env=setup.env,
                    # start collecting counters as soon as possible
                    on_started=(counter_management.start
                                if counter_management else None),
                    debug=browser_config['debug'],
                    debugger=browser_config['debugger'],
                    debugger_args=browser_config['debugger_args']
                )
            except Exception:
                self.check_for_crashes(browser_config, minidump_dir,
                                       test_config['name'])
                raise
            finally:
                if counter_management:
                    counter_management.stop()

            if test_config['mainthread']:
                rawlog = os.path.join(here, 'mainthread_io.log')
                if os.path.exists(rawlog):
                    processedlog = \
                        os.path.join(here, 'mainthread_io.json')
                    xre_path = \
                        os.path.dirname(browser_config['browser_path'])
                    mtio_py = os.path.join(here, 'mainthreadio.py')
                    command = ['python', mtio_py, rawlog,
                               processedlog, xre_path]
                    mtio = subprocess.Popen(command,
                                            env=os.environ.copy(),
                                            stdout=subprocess.PIPE)
                    output, stderr = mtio.communicate()
                    for line in output.split('\n'):
                        if line.strip() == '':
                            continue

                        print(line)
                        mainthread_error_count += 1
                    mozfile.remove(rawlog)

            if test_config['cleanup']:
                # HACK: add the pid to support xperf where we require
                # the pid in post processing
                talosconfig.generateTalosConfig(command_args,
                                                browser_config,
                                                test_config,
                                                pid=pcontext.pid)
                subprocess.call(
                    [sys.executable] + test_config['cleanup'].split()
                )

            # For startup tests, we launch the browser multiple times
            # with the same profile
            for fname in ('sessionstore.js', '.parentlock',
                          'sessionstore.bak'):
                mozfile.remove(os.path.join(setup.profile_dir, fname))

            # check for xperf errors
            if os.path.exists(browser_config['error_filename']) or \
               mainthread_error_count > 0:
                raise TalosRegression(
                    'Talos has found a regression, if you have questions'
                    ' ask for help in irc on #perf'
                )

            # add the results from the browser output
            if not run_in_debug_mode(browser_config):
                test_results.add(
                    '\n'.join(pcontext.output),
                    counter_results=(counter_management.results()
                                     if counter_management
                                     else None)
                )

            if setup.gecko_profile:
                setup.gecko_profile.symbolicate(i)

            self.check_for_crashes(browser_config, minidump_dir,
                                   test_config['name'])

        # include global (cross-cycle) counters
        test_results.all_counter_results.extend(
            [{key: value} for key, value in global_counters.items()]
        )
        for c in test_results.all_counter_results:
            for key, value in c.items():
                LOG.debug('COUNTER %r: %s' % (key, value))

        # if running against a code-coverage instrumented build, move the
        # produced gcda files to a folder where they will be collected later
        if browser_config.get('code_coverage', False):
            setup.collect_or_clean_ccov()

        # return results
        return test_results



from __future__ import absolute_import

import os

here = os.path.realpath(__file__)
__TESTS_DIR = os.path.join(os.path.dirname(os.path.dirname(here)), 'tests')


def remove_develop_files(starting_dir=__TESTS_DIR):
    for file_name in os.listdir(starting_dir):

        file_path = os.path.join(starting_dir, file_name)

        if file_name.endswith('.develop') and os.path.isfile(file_path):
            os.remove(file_path)
        elif os.path.isdir(file_path):
            remove_develop_files(file_path)


def patched_build_manifest(config, manifestName):
    # for unit testing, we aren't downloading the tp5 pageset; therefore
    # manifest file doesn't exist; create a dummy one so we can continue
    if not os.path.exists(manifestName):
        try:
            if not os.path.exists(os.path.dirname(manifestName)):
                os.makedirs(os.path.dirname(manifestName))
            f = open(manifestName, "w")
            f.close()
        except Exception as e:
            raise(e)

    # read manifest lines
    with open(manifestName, 'r') as fHandle:
        manifestLines = fHandle.readlines()

    # write modified manifest lines
    with open(manifestName + '.develop', 'w') as newHandle:
        for line in manifestLines:
            newline = line.replace('localhost', config['webserver'])
            newline = newline.replace('page_load_test', 'tests')
            newHandle.write(newline)

    newManifestName = manifestName + '.develop'

    # return new manifest
    return newManifestName



from __future__ import absolute_import

import mozunit
import pytest

from talos.test import Test, TsBase, ts_paint
from talos.test import register_test
from talos.test import test_dict


class BasicTestA(Test):
    pass


class BasicTestB(Test):
    pass


class BasicTestC(Test):
    """basic description"""
    keys = [
        'nonnull_attrib',
        'null_attrib'
    ]

    nonnull_attrib = 'value'
    null_attrib = None


class NotATest(object):
    pass


class Test_register_test(object):

    def test_same_instance_returned(self):
        decorator = register_test()
        NewBasicTest = decorator(BasicTestA)

        assert BasicTestA is NewBasicTest

    def test_class_registered(self):
        _TESTS = test_dict()
        decorator = register_test()

        # class registered
        decorator(BasicTestB)
        assert 'BasicTestB' in _TESTS
        assert BasicTestB in _TESTS.values()

        # cannot register same class
        with pytest.raises(AssertionError):
            decorator(BasicTestB)

        # # cannot register other class type
        with pytest.raises(AssertionError):
            decorator(NotATest)


class TestTest(object):

    def test_same_class_name(self):
        assert BasicTestA.name() == 'BasicTestA'

    def test_class_doc(self):
        assert BasicTestA.description() is not None
        assert BasicTestC.description() == 'basic description'

    def test_init(self):
        basic_test = BasicTestA(new_attrib_a='value_a', new_attrib_b='value_b')
        assert basic_test.new_attrib_a == 'value_a'
        assert basic_test.new_attrib_b == 'value_b'

    def test_update(self):
        basic_test = BasicTestA()
        basic_test.update(new_attrib_a='value_a', new_attrib_b='value_b')

        assert basic_test.new_attrib_a == 'value_a'
        assert basic_test.new_attrib_b == 'value_b'

        basic_test.update(new_attrib_c='value_c')
        assert basic_test.new_attrib_c == 'value_c'

    def test_items(self):
        basic_test = BasicTestC()

        # returns iterable
        try:
            iter(basic_test.items())
        except TypeError:
            pytest.fail('Test.items() did not return iterator')

        tuple_result = basic_test.items()[0]
        assert len(tuple_result) == 2

        # returns not nones
        assert ('nonnull_attrib', 'value') in basic_test.items()
        assert ('null_attrib', None) not in basic_test.items()

        # not overriden Test instance
        test_instance = Test()
        assert test_instance.items() == [('name', 'Test')]

        # overriden Test instance
        test_instance = Test(unregistered_attr='value')
        assert ('unregistered_attr', 'value') not in test_instance.items()

        test_instance = Test()
        test_instance.update(keys=['cycles', 'desktop', 'lower_is_better'])
        assert dict(test_instance.items()) == {
            'name': 'Test', 'desktop': True, 'lower_is_better': True}

        test_instance = Test()
        test_instance.update(new_attrib='some')
        assert ('new_attrib', 'some') not in test_instance.items()

        test_instance = Test()
        test_instance.update(keys=['new_attrib'], new_attrib='value')
        assert dict(test_instance.items()) == {'name': 'Test', 'new_attrib': 'value'}

        test_instance = Test(cycles=20, desktop=False)
        assert test_instance.cycles == 20
        assert test_instance.desktop is False

        test_instance = Test()
        test_instance.update(cycles=20, desktop=False)
        assert test_instance.cycles == 20
        assert test_instance.desktop is False


class TestTsBase(object):
    ts_base_registered_keys = {
        'url',
        'url_timestamp',
        'timeout',
        'cycles',
        'profile_path',
        'gecko_profile',
        'gecko_profile_interval',
        'gecko_profile_entries',
        'gecko_profile_startup',
        'preferences',
        'xperf_counters',
        'xperf_providers',
        'xperf_user_providers',
        'xperf_stackwalk',
        'tpmozafterpaint',
        'fnbpaint',
        'tphero',
        'profile',
        'firstpaint',
        'userready',
        'testeventmap',
        'base_vs_ref',
        'extensions',
        'filters',
        'setup',
        'cleanup',
        'webextensions',
        'reinstall',
    }

    def setup_method(self):
        self.test_instance = TsBase()

    def test_no_unknown_keys_are_somehow_added_alongside_registered_ones(self):
        assert set(self.test_instance.keys) == self.ts_base_registered_keys

        self.test_instance.update(attribute_one='value', attribute_two='value')
        assert set(self.test_instance.keys) == self.ts_base_registered_keys

    def test_nonnull_keys_show_up(self):
        assert dict(self.test_instance.items()) == {
            'name': 'TsBase',
            'filters': self.test_instance.filters
        }

        self.test_instance.update(timeout=500)
        assert dict(self.test_instance.items()) == {
            'name': 'TsBase',
            'filters': self.test_instance.filters,
            'timeout': 500
        }


class Test_ts_paint(object):
    def test_test_nonnull_keys_show_up(self):
        test_instance = ts_paint()
        keys = {key for key, _ in test_instance.items()}
        assert keys == {
            'name',
            'cycles',
            'timeout',
            'gecko_profile_startup',
            'gecko_profile_entries',
            'url',
            'xperf_counters',
            'filters',
            'tpmozafterpaint'
        }


if __name__ == '__main__':
    mozunit.main()



from __future__ import absolute_import

import os

import mock
import mozunit

from talos.ffsetup import FFSetup


class TestFFSetup(object):

    def setup_method(self, method):
        self.ffsetup = FFSetup(
            # browser_config
            {
                "env": {},
                "symbols_path": "",
                "preferences": {},
                "webserver": "",
                "extensions": []
            },
            # test_config
            {
                "preferences": {},
                "extensions": [],
                "profile": None
            }
        )

        # setup proxy logger

    def test_clean(self):
        # tmp dir removed
        assert self.ffsetup._tmp_dir is not None
        assert os.path.exists(self.ffsetup._tmp_dir) is True

        self.ffsetup.clean()

        assert self.ffsetup._tmp_dir is not None
        assert os.path.exists(self.ffsetup._tmp_dir) is False

        # gecko profile also cleaned
        gecko_profile = mock.Mock()
        self.ffsetup.gecko_profile = gecko_profile

        self.ffsetup.clean()

        assert gecko_profile.clean.called is True

#     def test_as_context_manager(self):
#         self.ffsetup._init_env = mock.Mock()
#         self.ffsetup._init_profile = mock.Mock()
#         self.ffsetup._run_profile = mock.Mock()
#         self.ffsetup._init_gecko_profile = mock.Mock()
#
#         with self.ffsetup as setup:
#             # env initiated
#             self.assertIsNotNone(setup.env)
#             # profile initiated
#             self.assertTrue(setup._init_profile.called)
#             # gecko profile initiated
#
#         # except raised
#         pass
#
#     def test_environment_init(self):
#         # self.env not empty
#         # browser_config env vars in self.env
#         # multiple calls return same self.env
#         pass
#
#     def test_profile_init(self):
#         # addons get installed
#         # webextensions get installed
#         # preferences contain interpolated values
#         # profile path is added
#         pass
#
#     def test_run_profile(self):
#         # exception raised
#         # browser process launched
#         pass
#
#     def test_gecko_profile_init(self):
#         # complains on not provided upload_dir
#         # self.gecko_profile not None
#         pass


if __name__ == '__main__':
    mozunit.main()



from __future__ import absolute_import

import copy
import os

import mock

import mozunit
import pytest
import conftest

from talos.config import (
    get_active_tests, get_test, get_config, get_browser_config,
    get_configs, ConfigurationError, DEFAULTS)
from talos.test import PageloaderTest

ORIGINAL_DEFAULTS = copy.deepcopy(DEFAULTS)


class mock_test(PageloaderTest):
    keys = ['tpmanifest', 'tpcycles', 'tppagecycles', 'tprender', 'tpchrome',
            'tpmozafterpaint', 'fnbpaint', 'tploadnocache', 'firstpaint', 'userready',
            'testeventmap', 'base_vs_ref', 'mainthread', 'resolution', 'cycles',
            'gecko_profile', 'gecko_profile_interval', 'gecko_profile_entries',
            'tptimeout', 'win_counters', 'w7_counters', 'linux_counters', 'mac_counters',
            'tpscrolltest', 'xperf_counters', 'timeout', 'shutdown', 'responsiveness',
            'profile_path', 'xperf_providers', 'xperf_user_providers', 'xperf_stackwalk',
            'format_pagename', 'filters', 'preferences', 'extensions', 'setup', 'cleanup',
            'lower_is_better', 'alert_threshold', 'unit', 'webextensions', 'profile',
            'tpmozafterpaint', 'url']

    tpmozafterpaint = 'value'
    firstpaint = 'value'
    userready = 'value'
    fnbpaint = 'value'


class Test_get_active_tests(object):
    def test_raises_exception_for_undefined_test(self):
        with pytest.raises(ConfigurationError):
            get_active_tests({'activeTests': 'undefined_test'})

        with pytest.raises(ConfigurationError):
            get_active_tests({'activeTests': '  undefined_test     '})

        with pytest.raises(ConfigurationError):
            get_active_tests({'activeTests': 'undef_test:undef_test2:undef_test3'})


class Test_get_test(object):
    global_overrides = {
        'tpmozafterpaint': 'overriden',
        'firstpaint': 'overriden',
        'userready': 'overriden',
        'fnbpaint': 'overriden'
    }

    config = {'webserver': 'test_webserver'}

    def test_doesnt_override_specific_keys_unless_they_are_null(self):
        test_instance = mock_test()
        test_dict = get_test({}, self.global_overrides, [], test_instance)

        assert test_dict['tpmozafterpaint'] == 'value'
        assert test_dict['firstpaint'] == 'value'
        assert test_dict['userready'] == 'value'
        assert test_dict['fnbpaint'] == 'value'

        # nulls still get overriden
        test_instance = mock_test(
            tpmozafterpaint=None, firstpaint=None, userready=None, fnbpaint=None)
        test_dict = get_test({}, self.global_overrides, [], test_instance)

        assert test_dict['tpmozafterpaint'] == 'overriden'
        assert test_dict['firstpaint'] == 'overriden'
        assert test_dict['userready'] == 'overriden'
        assert test_dict['fnbpaint'] == 'overriden'

    @mock.patch('talos.config.open', create=True)
    def test_interpolate_keys(self, mock_open):
        mock_open.return_value = mock.MagicMock(readlines=lambda: [])

        test_instance = mock_test(url='${talos}/test_page.html',
                                  tpmanifest='${talos}/file.manifest')

        test_dict = get_test(self.config, self.global_overrides, [], test_instance)
        assert test_dict['url'].startswith('http://test_webserver/')
        assert '${talos}' not in test_dict['url']
        assert '${talos}' not in test_dict['tpmanifest']

    def test_build_tpmanifest(self, tmpdir):
        manifest_file = tmpdir.join('file.manifest').ensure(file=True)
        test_instance = mock_test(url='test_page.html',
                                  tpmanifest=str(manifest_file))

        test_dict = get_test(self.config, self.global_overrides, [], test_instance)
        assert test_dict['tpmanifest'].endswith('.develop')

    def test_add_counters(self):
        test_instance = mock_test(
            linux_counters=None,
            mac_counters=[],
            win_counters=['counter_a'],
            w7_counters=['counter_a', 'counter_b'],
            xperf_counters=['counter_a', 'counter_extra']
        )

        counters = ['counter_a', 'counter_b', 'counter_c']
        test_dict = get_test(
            self.config, self.global_overrides, counters, test_instance)

        assert test_dict['linux_counters'] == counters
        assert test_dict['mac_counters'] == counters
        assert test_dict['win_counters'] == counters
        assert test_dict['w7_counters'] == counters
        assert set(test_dict['xperf_counters']) == set(counters + ['counter_extra'])


class Test_get_browser_config(object):
    required = ('extensions', 'browser_path', 'browser_wait',
                'extra_args', 'buildid', 'env', 'init_url', 'webserver')
    optional = ['bcontroller_config',
                'child_process',
                'debug',
                'debugger',
                'debugger_args',
                'develop',
                'e10s',
                'process',
                'framework',
                'repository',
                'sourcestamp',
                'symbols_path',
                'test_timeout',
                'xperf_path',
                'error_filename',
                'no_upload_results',
                'stylothreads',
                'subtests',
                'preferences']

    def test_that_contains_title(self):
        config_no_optionals = dict.fromkeys(self.required, '')
        config_no_optionals.update(title='is_mandatory')

        browser_config = get_browser_config(config_no_optionals)
        assert browser_config['title'] == 'is_mandatory'

    def test_raises_keyerror_for_missing_title(self):
        config_missing_title = dict.fromkeys(self.required, '')

        with pytest.raises(KeyError):
            get_browser_config(config_missing_title)

    def test_raises_keyerror_for_required_keys(self):
        config_missing_required = dict.fromkeys(self.required, '')
        config_missing_required.update(title='is_mandatory')
        del config_missing_required['extensions']

        with pytest.raises(KeyError):
            get_browser_config(config_missing_required)

    def test_doesnt_raise_on_missing_optionals(self):
        config_missing_optionals = dict.fromkeys(self.required, '')
        config_missing_optionals['title'] = 'is_mandatory'

        try:
            get_browser_config(config_missing_optionals)
        except KeyError:
            pytest.fail('Must not raise exception on missing optional')


class Test_get_config(object):
    @classmethod
    def setup_class(cls):
        cls.argv = '--suite other-e10s --mainthread -e /some/random/path'.split()
        cls.argv_unprovided_tests = '-e /some/random/path'.split()
        cls.argv_unknown_suite = '--suite random-unknown-suite -e /some/random/path'.split()
        cls.argv_overrides_defaults = '''
        --suite other-e10s
        --executablePath /some/random/path
        --cycles 20
        --geckoProfile
        --geckoProfileInterval 1000
        --geckoProfileEntries 1000
        --mainthread
        --tpcycles 20
        --mozAfterPaint
        --firstPaint
        --firstNonBlankPaint
        --userReady
        --tppagecycles 20
        '''.split()

        cls.argv_ts_paint = '--activeTests ts_paint -e /some/random/path'.split()
        cls.argv_ts_paint_webext = '--activeTests ts_paint_webext -e /some/random/path'.split()
        cls.argv_ts_paint_heavy = '--activeTests ts_paint_heavy -e /some/random/path'.split()
        cls.argv_sessionrestore = '--activeTests sessionrestore -e /some/random/path'.split()
        cls.argv_sessionrestore_no_auto_restore = \
            '--activeTests sessionrestore_no_auto_restore -e /some/random/path'.split()
        cls.argv_sessionrestore_many_windows = \
            '--activeTests sessionrestore_many_windows -e /some/random/path'.split()
        cls.argv_tresize = '--activeTests tresize -e /some/random/path'.split()
        cls.argv_cpstartup = '--activeTests cpstartup -e /some/random/path'.split()
        cls.argv_tabpaint = '--activeTests tabpaint -e /some/random/path'.split()
        cls.argv_tabswitch = '--activeTests tabswitch -e /some/random/path'.split()
        cls.argv_tart = '--activeTests tart -e /some/random/path'.split()
        cls.argv_damp = '--activeTests damp -e /some/random/path'.split()
        cls.argv_glterrain = '--activeTests glterrain -e /some/random/path'.split()
        cls.argv_glvideo = '--activeTests glvideo -e /some/random/path'.split()
        cls.argv_tp5n = '--activeTests tp5n -e /some/random/path'.split()
        cls.argv_tp5o = '--activeTests tp5o -e /some/random/path'.split()
        cls.argv_tp5o_webext = '--activeTests tp5o_webext -e /some/random/path'.split()
        cls.argv_tp5o_scroll = '--activeTests tp5o_scroll -e /some/random/path'.split()
        cls.argv_v8_7 = '--activeTests v8_7 -e /some/random/path'.split()
        cls.argv_kraken = '--activeTests kraken -e /some/random/path'.split()
        cls.argv_basic_compositor_video = \
            '--activeTests basic_compositor_video -e /some/random/path'.split()
        cls.argv_dromaeo_css = '--activeTests dromaeo_css -e /some/random/path'.split()
        cls.argv_dromaeo_dom = '--activeTests dromaeo_dom -e /some/random/path'.split()
        cls.argv_tsvgm = '--activeTests tsvgm -e /some/random/path'.split()
        cls.argv_tsvgx = '--activeTests tsvgx -e /some/random/path'.split()
        cls.argv_tsvg_static = '--activeTests tsvg_static -e /some/random/path'.split()
        cls.argv_tsvgr_opacity = '--activeTests tsvgr_opacity -e /some/random/path'.split()
        cls.argv_tscrollx = '--activeTests tscrollx -e /some/random/path'.split()
        cls.argv_a11yr = '--activeTests a11yr -e /some/random/path'.split()
        cls.argv_perf_reftest = '--activeTests perf_reftest -e /some/random/path'.split()
        cls.argv_perf_reftest_singletons = \
            '--activeTests perf_reftest_singletons -e /some/random/path'.split()

    @classmethod
    def teardown_class(cls):
        conftest.remove_develop_files()

    def test_correctly_overrides_test_valus(self):
        config = get_config(self.argv)
        assert bool(config) is True

        # no null values
        null_keys = [key for key, val in config.iteritems() if val is None]
        assert len(null_keys) == 0

        # expected keys are there
        assert config['browser_path'] == '/some/random/path'
        assert config['suite'] == 'other-e10s'
        assert config['mainthread'] is True

        # default values overriden
        config = get_config(self.argv_overrides_defaults)
        assert config['basetest'] == ORIGINAL_DEFAULTS['basetest']

    def test_config_has_tests(self):
        config = get_config(self.argv)
        assert len(config['tests']) > 0

    def test_global_variable_isnt_modified(self):
        get_config(self.argv)
        assert ORIGINAL_DEFAULTS == DEFAULTS

    def test_raises_except_if_unprovided_tests_on_cli(self):
        with pytest.raises(ConfigurationError):
            get_config(self.argv_unprovided_tests)

        with pytest.raises(ConfigurationError):
            get_config(self.argv_unknown_suite)

    def test_ts_paint_has_expected_attributes(self):
        config = get_config(self.argv_ts_paint)
        test_config = config['tests'][0]

        assert test_config['name'] == 'ts_paint'
        assert test_config['cycles'] == 20
        assert test_config['timeout'] == 150
        assert test_config['gecko_profile_startup'] is True
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['url'] != 'startup_test/tspaint_test.html'  # interpolation was done
        assert test_config['xperf_counters'] == []
        # TODO: these don't work; is this a bug?
        # assert test_config['win7_counters'] == []
        assert test_config['filters'] is not None
        assert test_config['tpmozafterpaint'] is True
        # assert test_config['mainthread'] is False
        # assert test_config['responsiveness'] is False
        # assert test_config['unit'] == 'ms'

    def test_ts_paint_webext_has_expected_attributes(self):
        config = get_config(self.argv_ts_paint_webext)
        test_config = config['tests'][0]

        assert test_config['name'] == 'ts_paint_webext'
        assert test_config['cycles'] == 20
        assert test_config['timeout'] == 150
        assert test_config['gecko_profile_startup'] is True
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['url'] != 'startup_test/tspaint_test.html'  # interpolation was done
        assert test_config['xperf_counters'] == []
        # TODO: these don't work; is this a bug?
        # assert test_config['win7_counters'] == []
        assert test_config['filters'] is not None
        assert test_config['tpmozafterpaint'] is True
        # assert test_config['mainthread'] is False
        # assert test_config['responsiveness'] is False
        # assert test_config['unit'] == 'ms'
        # TODO: this isn't overriden
        # assert test_config['webextensions'] != '${talos}/webextensions/dummy/dummy-signed.xpi'
        assert test_config['preferences'] == {'xpinstall.signatures.required': False}

    def test_ts_paint_heavy_has_expected_attributes(self):
        config = get_config(self.argv_ts_paint_heavy)
        test_config = config['tests'][0]

        assert test_config['name'] == 'ts_paint_heavy'
        assert test_config['cycles'] == 20
        assert test_config['timeout'] == 150
        assert test_config['gecko_profile_startup'] is True
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['url'] != 'startup_test/tspaint_test.html'  # interpolation was done
        assert test_config['xperf_counters'] == []
        # TODO: this doesn't work; is this a bug?
        # assert test_config['win7_counters'] == []
        assert test_config['filters'] is not None
        assert test_config['tpmozafterpaint'] is True
        # assert test_config['mainthread'] is False
        # assert test_config['responsiveness'] is False
        # assert test_config['unit'] == 'ms'
        assert test_config['profile'] == 'simple'

    def test_sessionrestore_has_expected_attributes(self):
        config = get_config(self.argv_sessionrestore)
        test_config = config['tests'][0]

        assert test_config['name'] == 'sessionrestore'
        assert test_config['cycles'] == 10
        assert test_config['timeout'] == 900
        assert test_config['gecko_profile_startup'] is True
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['reinstall'] == [
            'sessionstore.jsonlz4', 'sessionstore.js', 'sessionCheckpoints.json']
        assert test_config['url'] == 'about:home'
        assert test_config['preferences'] == {'browser.startup.page': 3}
        # assert test_config['unit'] == 'ms'

    def test_sesssionrestore_no_auto_restore_has_expected_attributes(self):
        config = get_config(self.argv_sessionrestore_no_auto_restore)
        test_config = config['tests'][0]

        assert test_config['name'] == 'sessionrestore_no_auto_restore'
        assert test_config['cycles'] == 10
        assert test_config['timeout'] == 900
        assert test_config['gecko_profile_startup'] is True
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['reinstall'] == [
            'sessionstore.jsonlz4', 'sessionstore.js', 'sessionCheckpoints.json']
        assert test_config['url'] == 'about:home'
        assert test_config['preferences'] == {'browser.startup.page': 1}
        # assert test_config['unit'] == 'ms'

    def test_sessionrestore_many_windows_has_expected_attributes(self):
        config = get_config(self.argv_sessionrestore_many_windows)
        test_config = config['tests'][0]

        assert test_config['name'] == 'sessionrestore_many_windows'
        assert test_config['cycles'] == 10
        assert test_config['timeout'] == 900
        assert test_config['gecko_profile_startup'] is True
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['reinstall'] == [
            'sessionstore.jsonlz4', 'sessionstore.js', 'sessionCheckpoints.json']
        assert test_config['url'] == 'about:home'
        assert test_config['preferences'] == {'browser.startup.page': 3}
        # assert test_config['unit'] == 'ms'

    def test_tresize_has_expected_attributes(self):
        config = get_config(self.argv_tresize)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tresize'
        assert test_config['cycles'] == 20
        assert test_config['url'] != 'startup_test/tresize/addon/content/tresize-test.html'
        assert test_config['timeout'] == 150
        assert test_config['gecko_profile_interval'] == 2
        assert test_config['gecko_profile_entries'] == 1000000
        assert test_config['tpmozafterpaint'] is True
        assert test_config['filters'] is not None
        # assert test_config['unit'] == 'ms'

    def test_cpstartup_has_expected_attributes(self):
        config = get_config(self.argv_cpstartup)
        test_config = config['tests'][0]

        assert test_config['name'] == 'cpstartup'
        assert test_config['tpcycles'] == 1
        assert test_config['tpmanifest'] != '${talos}/tests/cpstartup/cpstartup.manifest'
        assert test_config['tppagecycles'] == 20
        assert test_config['gecko_profile_entries'] == 1000000
        assert test_config['tploadnocache'] is True
        assert test_config['unit'] == 'ms'
        assert test_config['preferences'] == {
            'addon.test.cpstartup.webserver': '${webserver}',
            'browser.link.open_newwindow': 3,
            'browser.link.open_newwindow.restriction': 2,
        }

    def test_tabpaint_has_expected_attributes(self):
        config = get_config(self.argv_tabpaint)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tabpaint'
        assert test_config['tpcycles'] == 1
        assert test_config['tpmanifest'] != '${talos}/tests/tabpaint/tabpaint.manifest'
        assert test_config['tppagecycles'] == 20
        assert test_config['gecko_profile_entries'] == 1000000
        assert test_config['tploadnocache'] is True
        assert test_config['unit'] == 'ms'
        assert test_config['preferences'] == {
            'browser.link.open_newwindow': 3,
            'browser.link.open_newwindow.restriction': 2,
        }

    def test_tabswitch_has_expected_attributes(self):
        config = get_config(self.argv_tabswitch)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tabswitch'
        assert test_config['tpcycles'] == 1
        assert test_config['tpmanifest'] != '${talos}/tests/tabswitch/tabswitch.manifest'
        assert test_config['tppagecycles'] == 5
        assert test_config['gecko_profile_entries'] == 5000000
        assert test_config['tploadnocache'] is True
        assert test_config['preferences'] == {
            'addon.test.tabswitch.urlfile': os.path.join('${talos}',
                                                         'tests',
                                                         'tp5o.html'),
            'addon.test.tabswitch.webserver': '${webserver}',
            'addon.test.tabswitch.maxurls': -1,
        }
        assert test_config['unit'] == 'ms'

    def test_tart_has_expected_attributes(self):
        config = get_config(self.argv_tart)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tart'
        assert test_config['tpmanifest'] != '${talos}/tests/tart/tart.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 25
        assert test_config['tploadnocache'] is True
        assert test_config['tpmozafterpaint'] is False
        assert test_config['gecko_profile_interval'] == 10
        assert test_config['gecko_profile_entries'] == 1000000
        assert 'win_counters' not in test_config
        assert 'w7_counters' not in test_config
        assert 'linux_counters' not in test_config
        assert 'mac_counters' not in test_config
        assert test_config['preferences'] == {
            'layout.frame_rate': 0,
            'docshell.event_starvation_delay_hint': 1,
            'dom.send_after_paint_to_content': False
        }
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms'

    def test_damp_has_expected_attributes(self):
        config = get_config(self.argv_damp)
        test_config = config['tests'][0]

        assert test_config['name'] == 'damp'
        assert test_config['tpmanifest'] != '${talos}/tests/devtools/damp.manifest'
        assert test_config['cycles'] == 5
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 5
        assert test_config['tploadnocache'] is True
        assert test_config['tpmozafterpaint'] is False
        assert test_config['gecko_profile_interval'] == 10
        assert test_config['gecko_profile_entries'] == 1000000
        assert 'win_counters' not in test_config
        assert 'w7_counters' not in test_config
        assert 'linux_counters' not in test_config
        assert 'mac_counters' not in test_config
        assert test_config['filters'] is not None
        assert test_config['preferences'] == {
            'devtools.memory.enabled': True,
            'addon.test.damp.webserver': '${webserver}'
        }
        assert test_config['unit'] == 'ms'

    def test_glterrain_has_expected_attributes(self):
        config = get_config(self.argv_glterrain)
        test_config = config['tests'][0]

        assert test_config['name'] == 'glterrain'
        assert test_config['tpmanifest'] != '${talos}/tests/webgl/glterrain.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 25
        assert test_config['tploadnocache'] is True
        assert test_config['tpmozafterpaint'] is False
        assert test_config['tpchrome'] is False
        assert test_config['gecko_profile_interval'] == 10
        assert test_config['gecko_profile_entries'] == 2000000
        assert 'win_counters' not in test_config
        assert 'w7_counters' not in test_config
        assert 'linux_counters' not in test_config
        assert 'mac_counters' not in test_config
        assert test_config['preferences'] == {
            'layout.frame_rate': 0,
            'docshell.event_starvation_delay_hint': 1,
            'dom.send_after_paint_to_content': False
        }
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'frame interval'

    def test_glvideo_has_expected_attributes(self):
        config = get_config(self.argv_glvideo)
        test_config = config['tests'][0]

        assert test_config['name'] == 'glvideo'
        assert test_config['tpmanifest'] != '${talos}/tests/webgl/glvideo.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 5
        assert test_config['tploadnocache'] is True
        assert test_config['tpmozafterpaint'] is False
        assert test_config['tpchrome'] is False
        assert test_config['gecko_profile_interval'] == 2
        assert test_config['gecko_profile_entries'] == 2000000
        assert 'win_counters' not in test_config
        assert 'w7_counters' not in test_config
        assert 'linux_counters' not in test_config
        assert 'mac_counters' not in test_config
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms'

    @mock.patch('talos.config.build_manifest', conftest.patched_build_manifest)
    def test_tp5n_has_expected_attributes(self):
        config = get_config(self.argv_tp5n)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tp5n'
        assert test_config['resolution'] == 20
        assert test_config['tpmanifest'] != '${talos}/tests/tp5n/tp5n.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 1
        assert test_config['cycles'] == 1
        assert test_config['tpmozafterpaint'] is True
        assert test_config['tptimeout'] == 5000
        assert test_config['mainthread'] is True
        assert test_config['w7_counters'] == []
        assert test_config['win_counters'] == []
        assert test_config['linux_counters'] == []
        assert test_config['mac_counters'] == []
        assert test_config['xperf_counters'] == [
            'main_startup_fileio', 'main_startup_netio',
            'main_normal_fileio', 'main_normal_netio',
            'nonmain_startup_fileio', 'nonmain_normal_fileio',
            'nonmain_normal_netio', 'mainthread_readcount',
            'mainthread_readbytes', 'mainthread_writecount',
            'mainthread_writebytes'
        ]
        assert test_config['xperf_providers'] == [
            'PROC_THREAD', 'LOADER', 'HARD_FAULTS', 'FILENAME',
            'FILE_IO', 'FILE_IO_INIT'
        ]
        assert test_config['xperf_user_providers'] == [
            'Mozilla Generic Provider',
            'Microsoft-Windows-TCPIP'
        ]
        assert test_config['xperf_stackwalk'] == [
            'FileCreate', 'FileRead', 'FileWrite', 'FileFlush',
            'FileClose'
        ]
        assert test_config['filters'] is not None
        assert test_config['timeout'] == 1800
        assert test_config['preferences'] == {
            'extensions.enabledScopes': '',
            'talos.logfile': 'browser_output.txt'
        }
        assert test_config['unit'] == 'ms'

    @mock.patch('talos.config.build_manifest', conftest.patched_build_manifest)
    def test_tp5o_has_expected_attributes(self):
        config = get_config(self.argv_tp5o)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tp5o'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 25
        assert test_config['cycles'] == 1
        assert test_config['tpmozafterpaint'] is True
        assert test_config['tptimeout'] == 5000
        assert test_config['mainthread'] is False
        assert test_config['tpmanifest'] != '${talos}/tests/tp5n/tp5o.manifest'
        assert test_config['win_counters'] == ['% Processor Time']
        assert test_config['w7_counters'] == ['% Processor Time']
        assert test_config['linux_counters'] == ['XRes']
        assert test_config['mac_counters'] == []
        assert test_config['responsiveness'] is True
        assert test_config['gecko_profile_interval'] == 2
        assert test_config['gecko_profile_entries'] == 4000000
        assert test_config['filters'] is not None
        assert test_config['timeout'] == 1800
        assert test_config['unit'] == 'ms'

    @mock.patch('talos.config.build_manifest', conftest.patched_build_manifest)
    def test_tp5o_webext_has_expected_attributes(self):
        config = get_config(self.argv_tp5o_webext)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tp5o_webext'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 25
        assert test_config['cycles'] == 1
        assert test_config['tpmozafterpaint'] is True
        assert test_config['tptimeout'] == 5000
        assert test_config['mainthread'] is False
        assert test_config['tpmanifest'] != '${talos}/tests/tp5n/tp5o.manifest'
        assert test_config['win_counters'] == ['% Processor Time']
        assert test_config['w7_counters'] == ['% Processor Time']
        assert test_config['linux_counters'] == ['XRes']
        assert test_config['mac_counters'] == []
        assert test_config['responsiveness'] is True
        assert test_config['gecko_profile_interval'] == 2
        assert test_config['gecko_profile_entries'] == 4000000
        assert test_config['filters'] is not None
        assert test_config['timeout'] == 1800
        assert test_config['unit'] == 'ms'
        assert test_config['webextensions'] == '${talos}/webextensions/dummy/dummy.xpi'
        assert test_config['preferences'] == {'xpinstall.signatures.required': False}

    @mock.patch('talos.config.build_manifest', conftest.patched_build_manifest)
    def test_tp5o_scroll_has_expected_attributes(self):
        config = get_config(self.argv_tp5o_scroll)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tp5o_scroll'
        assert test_config['tpmanifest'] != '${talos}/tests/tp5n/tp5o.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 12
        assert test_config['gecko_profile_interval'] == 2
        assert test_config['gecko_profile_entries'] == 2000000
        assert test_config['tpscrolltest'] is True
        assert test_config['tpmozafterpaint'] is False
        assert test_config['preferences'] == {
            'layout.frame_rate': 0,
            'docshell.event_starvation_delay_hint': 1,
            'dom.send_after_paint_to_content': False,
            'layout.css.scroll-behavior.spring-constant': "'10'",
            'toolkit.framesRecording.bufferSize': 10000
        }
        assert test_config['filters'] is not None
        assert test_config['unit'] == '1/FPS'

    def test_v8_7_has_expected_attributes(self):
        config = get_config(self.argv_v8_7)
        test_config = config['tests'][0]

        assert test_config['name'] == 'v8_7'
        assert test_config['tpmanifest'] != '${talos}/tests/v8_7/v8.manifest'
        assert test_config['gecko_profile_interval'] == 1
        assert test_config['gecko_profile_entries'] == 1000000
        assert test_config['tpcycles'] == 1
        assert test_config['resolution'] == 20
        assert test_config['tpmozafterpaint'] is False
        assert test_config['preferences'] == {'dom.send_after_paint_to_content': False}
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'score'
        assert test_config['lower_is_better'] is False

    def test_kraken_has_expected_attributes(self):
        config = get_config(self.argv_kraken)
        test_config = config['tests'][0]

        assert test_config['name'] == 'kraken'
        assert test_config['tpmanifest'] != '${talos}/tests/kraken/kraken.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 1
        assert test_config['gecko_profile_interval'] == 1
        assert test_config['gecko_profile_entries'] == 5000000
        assert test_config['tpmozafterpaint'] is False
        assert test_config['tpchrome'] is False
        assert test_config['preferences'] == {'dom.send_after_paint_to_content': False}
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'score'

    def test_basic_compositor_video_has_expected_attributes(self):
        config = get_config(self.argv_basic_compositor_video)
        test_config = config['tests'][0]

        assert test_config['name'] == 'basic_compositor_video'
        assert test_config['tpmanifest'] != '${talos}/tests/video/video.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 12
        assert test_config['tpchrome'] is False
        assert test_config['timeout'] == 10000
        assert test_config['gecko_profile_interval'] == 1
        assert test_config['gecko_profile_entries'] == 2000000
        assert test_config['preferences'] == {
            'full-screen-api.allow-trusted-requests-only': False,
            'layers.acceleration.force-enabled': False,
            'layers.acceleration.disabled': True,
            'layout.frame_rate': 0,
            'docshell.event_starvation_delay_hint': 1,
            'full-screen-api.warning.timeout': 500,
            'media.ruin-av-sync.enabled': True
        }
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms/frame'
        assert test_config['lower_is_better'] is True

    def test_dromaeo_css_has_expected_attributes(self):
        config = get_config(self.argv_dromaeo_css)
        test_config = config['tests'][0]

        assert test_config['name'] == 'dromaeo_css'
        assert test_config['tpcycles'] == 1
        assert test_config['filters'] is not None
        assert test_config['lower_is_better'] is False
        assert test_config['alert_threshold'] == 5.0
        assert test_config['tpchrome'] is False
        assert test_config['gecko_profile_interval'] == 2
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['tpmanifest'] != '${talos}/tests/dromaeo/css.manifest'
        assert test_config['unit'] == 'score'

    def test_dromaeo_dom_has_expected_attributes(self):
        config = get_config(self.argv_dromaeo_dom)
        test_config = config['tests'][0]

        assert test_config['name'] == 'dromaeo_dom'
        assert test_config['tpcycles'] == 1
        assert test_config['filters'] is not None
        assert test_config['lower_is_better'] is False
        assert test_config['alert_threshold'] == 5.0
        assert test_config['tpchrome'] is False
        assert test_config['gecko_profile_interval'] == 2
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['tpmanifest'] != '${talos}/tests/dromaeo/dom.manifest'
        assert test_config['unit'] == 'score'

    def test_tsvgm_has_expected_attributes(self):
        config = get_config(self.argv_tsvgm)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tsvgm'
        assert test_config['tpmanifest'] != '${talos}/tests/svgx/svgm.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 7
        assert test_config['tpmozafterpaint'] is False
        assert test_config['tpchrome'] is False
        assert test_config['gecko_profile_interval'] == 10
        assert test_config['gecko_profile_entries'] == 1000000
        assert test_config['preferences'] == {
            'layout.frame_rate': 0,
            'docshell.event_starvation_delay_hint': 1,
            'dom.send_after_paint_to_content': False
        }
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms'

    def test_tsvgx_has_expected_attributes(self):
        config = get_config(self.argv_tsvgx)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tsvgx'
        assert test_config['tpmanifest'] != '${talos}/tests/svgx/svgx.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 25
        assert test_config['tpmozafterpaint'] is False
        assert test_config['tpchrome'] is False
        assert test_config['gecko_profile_interval'] == 10
        assert test_config['gecko_profile_entries'] == 1000000
        assert test_config['preferences'] == {
            'layout.frame_rate': 0,
            'docshell.event_starvation_delay_hint': 1,
            'dom.send_after_paint_to_content': False
        }
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms'

    def test_tsvg_static_has_expected_attributes(self):
        config = get_config(self.argv_tsvg_static)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tsvg_static'
        assert test_config['tpmanifest'] != '${talos}/tests/svg_static/svg_static.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 25
        assert test_config['tpmozafterpaint'] is True
        assert test_config['tpchrome'] is False
        assert test_config['gecko_profile_interval'] == 1
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms'

    def test_tsvgr_opacity_has_expected_attributes(self):
        config = get_config(self.argv_tsvgr_opacity)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tsvgr_opacity'
        assert test_config['tpmanifest'] != '${talos}/tests/svg_opacity/svg_opacity.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 25
        assert test_config['tpmozafterpaint'] is True
        assert test_config['tpchrome'] is False
        assert test_config['gecko_profile_interval'] == 1
        assert test_config['gecko_profile_entries'] == 10000000
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms'

    def test_tscrollx_has_expected_attributes(self):
        config = get_config(self.argv_tscrollx)
        test_config = config['tests'][0]

        assert test_config['name'] == 'tscrollx'
        assert test_config['tpmanifest'] != '${talos}/tests/scroll/scroll.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 25
        assert test_config['tpmozafterpaint'] is False
        assert test_config['tpchrome'] is False
        assert test_config['gecko_profile_interval'] == 1
        assert test_config['gecko_profile_entries'] == 1000000
        assert test_config['preferences'] == {
            'layout.frame_rate': 0,
            'docshell.event_starvation_delay_hint': 1,
            'dom.send_after_paint_to_content': False,
            'layout.css.scroll-behavior.spring-constant': "'10'",
            'toolkit.framesRecording.bufferSize': 10000
        }
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms'

    def test_a11yr_has_expect_attributes(self):
        config = get_config(self.argv_a11yr)
        test_config = config['tests'][0]

        assert test_config['name'] == 'a11yr'
        assert test_config['tpmanifest'] != '${talos}/tests/a11y/a11y.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 25
        assert test_config['tpmozafterpaint'] is True
        assert test_config['tpchrome'] is False
        assert test_config['preferences'] == {'dom.send_after_paint_to_content': False}
        assert test_config['unit'] == 'ms'
        assert test_config['alert_threshold'] == 5.0

    def test_perf_reftest_has_expected_attributes(self):
        config = get_config(self.argv_perf_reftest)
        test_config = config['tests'][0]

        assert test_config['name'] == 'perf_reftest'
        assert test_config['base_vs_ref'] is True
        assert test_config['tpmanifest'] != '${talos}/tests/perf-reftest/perf_reftest.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 10
        assert test_config['tptimeout'] == 30000
        assert test_config['gecko_profile_interval'] == 1
        assert test_config['gecko_profile_entries'] == 2000000
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms'
        assert test_config['lower_is_better'] is True
        assert test_config['alert_threshold'] == 5.0

    def test_perf_reftest_singletons_has_expected_attributes(self):
        config = get_config(self.argv_perf_reftest_singletons)
        test_config = config['tests'][0]

        assert test_config['name'] == 'perf_reftest_singletons'
        assert test_config['tpmanifest'] != \
            '${talos}/tests/perf-reftest-singletons/perf_reftest_singletons.manifest'
        assert test_config['tpcycles'] == 1
        assert test_config['tppagecycles'] == 15
        assert test_config['tptimeout'] == 30000
        assert test_config['gecko_profile_interval'] == 1
        assert test_config['gecko_profile_entries'] == 2000000
        assert test_config['filters'] is not None
        assert test_config['unit'] == 'ms'
        assert test_config['lower_is_better'] is True
        assert test_config['alert_threshold'] == 5.0


@mock.patch('talos.config.get_browser_config')
@mock.patch('talos.config.get_config')
def test_get_configs(get_config_mock, get_browser_config_mock):
    # unpacks in right order
    get_config_mock.return_value = 'first'
    get_browser_config_mock.return_value = 'second'

    first, second = get_configs()
    assert (first, second) == ('first', 'second')


if __name__ == '__main__':
    mozunit.main()
